@@ -11,6 +11,8 @@ test -z "$srcdir" && srcdir=.
     exit 1
 }
 
+pushd $srcdir
+
 # Fetch submodules if needed
 if test ! -f src/gvc/Makefile.am || test ! -f data/theme/gnome-shell-sass/COPYING;
 then
@@ -19,6 +21,8 @@ then
 fi
 git submodule update
 
+popd
+
 which gnome-autogen.sh || {
     echo "You need to install gnome-common from GNOME Git (or from"
     echo "your OS vendor's package manager)."
