@@ -297,7 +297,7 @@ const RemoteSearchProvider = new Lang.Class({
         // the provider is not compatible with the new version of the interface, launch
         // the app itself but warn so we can catch the error in logs
         log('Search provider ' + this.appInfo.get_id() + ' does not implement LaunchSearch');
-        this.appInfo.launch([], global.create_app_launch_context(0, -1));
+        this.appInfo.launch([], global.create_app_launch_context(0, -1), false);
     }
 });
 
